@@ -23,6 +23,9 @@
 #include "pango-modules.h"
 #include <string.h>
 
+#define PARAGRAPH_SEPARATOR 0x2029
+#define PARAGRAPH_SEPARATOR_STRING "\xE2\x80\xA9"
+  
 /* See http://www.unicode.org/unicode/reports/tr14/ if you hope
  * to understand the line breaking code.
  */
@@ -32,13 +35,14 @@ typedef enum
   BREAK_ALREADY_HANDLED,   /* didn't use the table */
   BREAK_PROHIBITED, /* no break, even if spaces intervene */
   BREAK_IF_SPACES,  /* "indirect break" (only if there are spaces) */
-  BREAK_ALLOWED,    /* "direct break" (can always break here) */
+  BREAK_ALLOWED     /* "direct break" (can always break here) */
   /* TR 14 has one more break-opportunity class,
    * "indirect break opportunity for combining marks following a space"
    * but we handle that inline in the code.
    */
 } BreakOpportunity;
 
+
 enum
 {
   INDEX_OPEN_PUNCTUATION,
@@ -77,10 +81,15 @@ enum
   INDEX_COMPLEX_CONTEXT,
   INDEX_AMBIGUOUS,
   INDEX_UNKNOWN,
-  INDEX_NEXT_LINE
+  INDEX_NEXT_LINE,
+  INDEX_HANGUL_L_JAMO,
+  INDEX_HANGUL_V_JAMO,
+  INDEX_HANGUL_T_JAMO,
+  INDEX_HANGUL_LV_SYLLABLE,
+  INDEX_HANGUL_LVT_SYLLABLE,
 };
 
-static BreakOpportunity row_OPEN_PUNCTUATION[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_OPEN_PUNCTUATION[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
@@ -89,7 +98,7 @@ static BreakOpportunity row_OPEN_PUNCTUATION[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_CLOSE_PUNCTUATION[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_CLOSE_PUNCTUATION[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_IF_SPACES, BREAK_ALLOWED, BREAK_ALLOWED,
@@ -98,7 +107,7 @@ static BreakOpportunity row_CLOSE_PUNCTUATION[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_QUOTATION[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_QUOTATION[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_IF_SPACES,
@@ -107,7 +116,7 @@ static BreakOpportunity row_QUOTATION[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_NON_BREAKING_GLUE[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_NON_BREAKING_GLUE[INDEX_END_OF_TABLE] = {
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_IF_SPACES,
@@ -116,7 +125,7 @@ static BreakOpportunity row_NON_BREAKING_GLUE[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_NON_STARTER[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_NON_STARTER[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED,
@@ -125,7 +134,7 @@ static BreakOpportunity row_NON_STARTER[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_EXCLAMATION[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_EXCLAMATION[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED,
@@ -134,7 +143,7 @@ static BreakOpportunity row_EXCLAMATION[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_SYMBOL[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_SYMBOL[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_IF_SPACES, BREAK_ALLOWED,
@@ -143,7 +152,7 @@ static BreakOpportunity row_SYMBOL[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_INFIX_SEPARATOR[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_INFIX_SEPARATOR[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_IF_SPACES, BREAK_IF_SPACES,
@@ -152,7 +161,7 @@ static BreakOpportunity row_INFIX_SEPARATOR[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_PREFIX[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_PREFIX[INDEX_END_OF_TABLE] = {
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_IF_SPACES, BREAK_IF_SPACES,
@@ -161,7 +170,7 @@ static BreakOpportunity row_PREFIX[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_POSTFIX[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_POSTFIX[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED,
@@ -170,7 +179,7 @@ static BreakOpportunity row_POSTFIX[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_NUMERIC[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_NUMERIC[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_IF_SPACES,
@@ -179,7 +188,7 @@ static BreakOpportunity row_NUMERIC[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_ALPHABETIC[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_ALPHABETIC[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_IF_SPACES, BREAK_IF_SPACES,
@@ -188,7 +197,7 @@ static BreakOpportunity row_ALPHABETIC[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_IDEOGRAPHIC[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_IDEOGRAPHIC[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_IF_SPACES, BREAK_ALLOWED, BREAK_ALLOWED,
@@ -197,7 +206,7 @@ static BreakOpportunity row_IDEOGRAPHIC[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_INSEPARABLE[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_INSEPARABLE[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED,
@@ -206,7 +215,7 @@ static BreakOpportunity row_INSEPARABLE[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_HYPHEN[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_HYPHEN[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_IF_SPACES, BREAK_ALLOWED,
@@ -215,7 +224,7 @@ static BreakOpportunity row_HYPHEN[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_AFTER[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_AFTER[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED,
@@ -224,7 +233,7 @@ static BreakOpportunity row_AFTER[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_BEFORE[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_BEFORE[INDEX_END_OF_TABLE] = {
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_IF_SPACES,
@@ -233,7 +242,7 @@ static BreakOpportunity row_BEFORE[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_BEFORE_AND_AFTER[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_BEFORE_AND_AFTER[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED,
@@ -242,7 +251,7 @@ static BreakOpportunity row_BEFORE_AND_AFTER[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_ZERO_WIDTH_SPACE[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_ZERO_WIDTH_SPACE[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED, BREAK_ALLOWED,
@@ -251,7 +260,7 @@ static BreakOpportunity row_ZERO_WIDTH_SPACE[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED
 };
 
-static BreakOpportunity row_COMBINING_MARK[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_COMBINING_MARK[INDEX_END_OF_TABLE] = {
   BREAK_ALLOWED, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_ALLOWED, BREAK_ALLOWED, BREAK_IF_SPACES, BREAK_IF_SPACES,
@@ -260,7 +269,7 @@ static BreakOpportunity row_COMBINING_MARK[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity row_WORD_JOINER[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity row_WORD_JOINER[INDEX_END_OF_TABLE] = {
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_IF_SPACES, BREAK_IF_SPACES,
   BREAK_IF_SPACES, BREAK_PROHIBITED, BREAK_PROHIBITED, BREAK_PROHIBITED,
   BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_IF_SPACES, BREAK_IF_SPACES,
@@ -269,7 +278,7 @@ static BreakOpportunity row_WORD_JOINER[INDEX_END_OF_TABLE] = {
   BREAK_PROHIBITED
 };
 
-static BreakOpportunity *line_break_rows[INDEX_END_OF_TABLE] = {
+static const BreakOpportunity *const line_break_rows[INDEX_END_OF_TABLE] = {
   row_OPEN_PUNCTUATION, /* INDEX_OPEN_PUNCTUATION */
   row_CLOSE_PUNCTUATION, /* INDEX_CLOSE_PUNCTUATION */
   row_QUOTATION, /* INDEX_QUOTATION */
@@ -294,7 +303,7 @@ static BreakOpportunity *line_break_rows[INDEX_END_OF_TABLE] = {
 };
 
 /* Map GUnicodeBreakType to table indexes */
-static int line_break_indexes[] = {
+static const int line_break_indexes[] = {
   INDEX_MANDATORY,
   INDEX_CARRIAGE_RETURN,
   INDEX_LINE_FEED,
@@ -325,9 +334,16 @@ static int line_break_indexes[] = {
   INDEX_AMBIGUOUS,
   INDEX_UNKNOWN,
   INDEX_NEXT_LINE,
-  INDEX_WORD_JOINER
+  INDEX_WORD_JOINER,
+  INDEX_HANGUL_L_JAMO,
+  INDEX_HANGUL_V_JAMO,
+  INDEX_HANGUL_T_JAMO,
+  INDEX_HANGUL_LV_SYLLABLE,
+  INDEX_HANGUL_LVT_SYLLABLE
 };
 
+#define BREAK_TYPE_SAFE(btype)            \
+         (btype < G_N_ELEMENTS(line_break_indexes) ? btype : G_UNICODE_BREAK_UNKNOWN)
 #define BREAK_INDEX(btype)                \
          (line_break_indexes[(btype)])
 #define BREAK_ROW(before_type)            \
@@ -335,14 +351,73 @@ static int line_break_indexes[] = {
 #define BREAK_OP(before_type, after_type) \
          (BREAK_ROW (before_type)[BREAK_INDEX (after_type)])
 #define IN_BREAK_TABLE(btype)             \
-         (btype <= G_N_ELEMENTS(line_break_indexes) && BREAK_INDEX(btype) < INDEX_END_OF_TABLE)
+         (btype < G_N_ELEMENTS(line_break_indexes) && BREAK_INDEX(btype) < INDEX_END_OF_TABLE)
+
+
+
+/*
+ * Hangul Conjoining Jamo handling.
+ *
+ * The way we implement it is just a bit different from TR14,
+ * but produces the same results.
+ * The same algorithm is also used in TR29 for cluster boundaries.
+ *
+ */
+
+
+/* An enum that works as the states of the Hangul syllables system.
+ **/
+typedef enum
+{
+  JAMO_L,	/* G_UNICODE_BREAK_HANGUL_L_JAMO */
+  JAMO_V,	/* G_UNICODE_BREAK_HANGUL_V_JAMO */
+  JAMO_T,	/* G_UNICODE_BREAK_HANGUL_T_JAMO */
+  JAMO_LV,	/* G_UNICODE_BREAK_HANGUL_LV_SYLLABLE */
+  JAMO_LVT,	/* G_UNICODE_BREAK_HANGUL_LVT_SYLLABLE */
+  NO_JAMO	/* Other */
+} JamoType;
+
+/* There are Hangul syllables encoded as characters, that act like a
+ * sequence of Jamos. For each character we define a JamoType
+ * that the character starts with, and one that it ends with.  This
+ * decomposes JAMO_LV and JAMO_LVT to simple other JAMOs.  So for
+ * example, a character with LineBreak type
+ * G_UNICODE_BREAK_HANGUL_LV_SYLLABLE has start=JAMO_L and end=JAMO_V.
+ */
+typedef struct _CharJamoProps
+{
+  JamoType start, end;
+} CharJamoProps;
+
+/* Map from JamoType to CharJamoProps that hold only simple
+ * JamoTypes (no LV or LVT) or none.
+ */
+const CharJamoProps HangulJamoProps[] = {
+  {JAMO_L, JAMO_L},	/* JAMO_L */
+  {JAMO_V, JAMO_V},	/* JAMO_V */
+  {JAMO_T, JAMO_T},	/* JAMO_T */
+  {JAMO_L, JAMO_V},	/* JAMO_LV */
+  {JAMO_L, JAMO_T},	/* JAMO_LVT */
+  {NO_JAMO, NO_JAMO}	/* NO_JAMO */
+};
+
+/* A character forms a syllable with the previous character if and only if:
+ * JamoType(this) is not NO_JAMO and:
+ *
+ * HangulJamoProps[JamoType(prev)].end and
+ * HangulJamoProps[JamoType(this)].start are equal,
+ * or the former is one less than the latter.
+ */
+
+#define IS_JAMO(btype)              \
+	((btype >= G_UNICODE_BREAK_HANGUL_L_JAMO) && \
+	 (btype <= G_UNICODE_BREAK_HANGUL_LVT_SYLLABLE))
+#define JAMO_TYPE(btype)      \
+	(IS_JAMO(btype) ? (btype - G_UNICODE_BREAK_HANGUL_L_JAMO) : NO_JAMO)
+
+
 
-/* Keep these in sync with the same macros in the test program */
 
-#define LEADING_JAMO(wc)  ((wc) >= 0x1100 && (wc) <= 0x115F)
-#define VOWEL_JAMO(wc)    ((wc) >= 0x1160 && (wc) <= 0x11A2)
-#define TRAILING_JAMO(wc) ((wc) >= 0x11A8 && (wc) <= 0x11F9)
-#define JAMO(wc)          ((wc) >= 0x1100 && (wc) <= 0x11FF)
 /* "virama script" is just an optimization; it includes a bunch of
  * scripts without viramas in them
  */
@@ -419,7 +494,7 @@ typedef enum
  * engine overrides it. Normally you should use pango_break()
  * instead; this function is mostly useful for chaining up
  * from a language engine override. Unlike pango_break(),
- * @analysis can be NULL, but only do that if you know what
+ * @analysis can be %NULL, but only do that if you know what
  * you're doing. (If you need an analysis to pass to pango_break(),
  * you need to pango_itemize() or use pango_get_log_attrs().)
  *
@@ -440,20 +515,26 @@ pango_default_break (const gchar   *text,
    */
 
   /* FIXME one cheesy optimization here would be to memset attrs to 0
-   * before we start, and then never assign FALSE to anything
+   * before we start, and then never assign %FALSE to anything
    */
 
   const gchar *next;
   gint i;
+
   gunichar prev_wc;
   gunichar next_wc;
+
+  JamoType prev_jamo;
+
   GUnicodeBreakType next_break_type;
   GUnicodeType prev_type;
   GUnicodeBreakType prev_break_type; /* skips spaces */
   gboolean prev_was_break_space;
+
   WordType current_word_type = WordNone;
   gunichar last_word_letter = 0;
   gunichar base_character = 0;
+
   SentenceState sentence_state = STATE_SENTENCE_OUTSIDE;
   /* Tracks what will be the end of the sentence if a period is
    * determined to actually be a sentence-ending period.
@@ -461,48 +542,44 @@ pango_default_break (const gchar   *text,
   gint possible_sentence_end = -1;
   /* possible sentence break before Open* after a period-ended sentence */
   gint possible_sentence_boundary = -1;
-  gint n_chars;
+  gboolean almost_done = FALSE;
+  gboolean done = FALSE;
 
   g_return_if_fail (text != NULL);
   g_return_if_fail (attrs != NULL);
 
-  n_chars = g_utf8_strlen (text, length);
-
   next = text;
   
-  /* + 1 because of the extra newline we stick on the end */
-  if (attrs_len < n_chars + 1)
-    {
-      g_warning ("pango_default_break(): the array of PangoLogAttr passed in must have at least N+1 elements, if there are N characters in the text being broken");
-      return;
-    }
-      
   prev_type = (GUnicodeType) -1;
   prev_break_type = G_UNICODE_BREAK_UNKNOWN;
   prev_was_break_space = FALSE;
   prev_wc = 0;
+  prev_jamo = NO_JAMO;
 
-  if (n_chars)
+  if (length == 0 || *text == '\0')
+    next_wc = PARAGRAPH_SEPARATOR;
+  else
     {
       next_wc = g_utf8_get_char (next);
       g_assert (next_wc != 0);
     }
-  else
-    next_wc = '\n';
 
   next_break_type = g_unichar_break_type (next_wc);
+  next_break_type = BREAK_TYPE_SAFE (next_break_type);
 
-  for (i = 0; i <= n_chars; i++)
+  for (i = 0; !done ; i++)
     {
       GUnicodeType type;
       gunichar wc;
       GUnicodeBreakType break_type;
       BreakOpportunity break_op;
+      JamoType jamo;
+      gboolean makes_hangul_syllable;
 
       wc = next_wc;
       break_type = next_break_type;
 
-      if (i == n_chars)
+      if (almost_done)
         {
           /*
            * If we have already reached the end of @text g_utf8_next_char()
@@ -510,17 +587,20 @@ pango_default_break (const gchar   *text,
            */
           next_wc = 0;
 	  next_break_type = G_UNICODE_BREAK_UNKNOWN;
+	  done = TRUE;
         }
       else
         {
           next = g_utf8_next_char (next);
 
-          if (i == n_chars - 1)
+	  if ((length >= 0 && next >= text + length) || *next == '\0')
             {
               /* This is how we fill in the last element (end position) of the
-               * attr array - assume there's a newline off the end of @text.
+               * attr array - assume there's a paragraph separators off the end
+	       * of @text.
                */
-              next_wc = '\n';
+	      next_wc = PARAGRAPH_SEPARATOR;
+	      almost_done = TRUE;
             }
           else
             {
@@ -529,9 +609,23 @@ pango_default_break (const gchar   *text,
             }
 	  
 	  next_break_type = g_unichar_break_type (next_wc);
+          next_break_type = BREAK_TYPE_SAFE (next_break_type);
         }
 
       type = g_unichar_type (wc);
+      jamo = JAMO_TYPE (break_type);
+
+      /* Determine wheter this forms a Hangul syllable with prev. */
+      if (jamo == NO_JAMO)
+        makes_hangul_syllable = FALSE;
+      else
+	{
+	  JamoType prev_end   = HangulJamoProps[prev_jamo].end  ;
+	  JamoType this_start = HangulJamoProps[     jamo].start;
+	  
+	  /* See comments before IS_JAMO */
+	  makes_hangul_syllable = (prev_end == this_start) || (prev_end + 1 == this_start);
+	}
 
       /* Can't just use the type here since isspace() doesn't
        * correspond to a Unicode character type
@@ -595,37 +689,21 @@ pango_default_break (const gchar   *text,
             case G_UNICODE_OTHER_LETTER:
             case G_UNICODE_TITLECASE_LETTER:
             case G_UNICODE_UPPERCASE_LETTER:
-              if (JAMO (wc))
-                {
-                  /* Break before Jamo if they are in a broken sequence or
-                   * next to non-Jamo, otherwise don't
-                   */
-                  if (LEADING_JAMO (wc) &&
-                      !LEADING_JAMO (prev_wc))
-                    attrs[i].is_cursor_position = TRUE;
-                  else if (VOWEL_JAMO (wc) &&
-                           !LEADING_JAMO (prev_wc) &&
-                           !VOWEL_JAMO (prev_wc))
-                    attrs[i].is_cursor_position = TRUE;
-                  else if (TRAILING_JAMO (wc) &&
-                           !LEADING_JAMO (prev_wc) &&
-                           !VOWEL_JAMO (prev_wc) &&
-                           !TRAILING_JAMO (prev_wc))
-                    attrs[i].is_cursor_position = TRUE;
-                  else
-                    attrs[i].is_cursor_position = FALSE;
-                }
+
+	      if (makes_hangul_syllable)
+                attrs[i].is_cursor_position = FALSE;
               else
                 {
-                  /* Handle non-Jamo non-combining chars */
+                  /* Handle non-Hangul-syllable non-combining chars */
 
-                  /* Break if preceded by Jamo; don't break if a
-                   * letter is preceded by a virama; break in all
-                   * other cases. No need to check whether we're
+                  /* Break before Jamo if they are in a broken sequence or
+                   * next to non-Jamo; break if preceded by Jamo; don't
+		   * break if a letter is preceded by a virama; break in
+		   * all other cases. No need to check whether we are or are
                    * preceded by Jamo explicitly, since a Jamo is not
                    * a virama, we just break in all cases where we
-                   * aren't preceded by a virama. Don't fool with viramas
-                   * if we aren't part of a script that uses them.
+                   * aren't a or preceded by a virama.  Don't fool with
+		   * viramas if we aren't part of a script that uses them.
                    */
 
                   if (VIRAMA_SCRIPT (wc))
@@ -685,6 +763,38 @@ pango_default_break (const gchar   *text,
            */
           attrs[i].is_char_break = TRUE;          
           
+	  /* Make any necessary replacements first */
+          switch (prev_break_type)
+            {
+	    case G_UNICODE_BREAK_HANGUL_L_JAMO:
+	    case G_UNICODE_BREAK_HANGUL_V_JAMO:
+	    case G_UNICODE_BREAK_HANGUL_T_JAMO:
+	    case G_UNICODE_BREAK_HANGUL_LV_SYLLABLE:
+	    case G_UNICODE_BREAK_HANGUL_LVT_SYLLABLE:
+	      /* treat Jamo as IDEOGRAPHIC from now
+	       */
+	      prev_break_type = G_UNICODE_BREAK_IDEOGRAPHIC;
+	      break;
+
+            case G_UNICODE_BREAK_AMBIGUOUS:
+	      /* FIXME
+               * we need to resolve the East Asian width
+               * to decide what to do here
+	       */
+            case G_UNICODE_BREAK_COMPLEX_CONTEXT:
+	      /* FIXME
+               * language engines should handle this case...
+	       */
+            case G_UNICODE_BREAK_UNKNOWN:
+	      /* convert unknown, complex, ambiguous to ALPHABETIC
+	       */
+	      prev_break_type = G_UNICODE_BREAK_ALPHABETIC;
+	      break;
+
+	    default:
+	      ;
+	    }
+
           switch (prev_break_type)
             {
             case G_UNICODE_BREAK_MANDATORY:
@@ -715,20 +825,8 @@ pango_default_break (const gchar   *text,
 	      g_assert_not_reached ();
               break;
 
-            case G_UNICODE_BREAK_AMBIGUOUS:
-              /* FIXME we need to resolve the East Asian width
-               * to decide what to do here
-               */
-            case G_UNICODE_BREAK_COMPLEX_CONTEXT:
-              /* FIXME language engines should handle this case... */
-            case G_UNICODE_BREAK_UNKNOWN:
-              /* treat unknown, complex, ambiguous as if they were
-               * alphabetic for now.
-               */
-              prev_break_type = G_UNICODE_BREAK_ALPHABETIC;
-              /* FALL THRU to use the pair table if appropriate */
-
             default:
+              g_assert (IN_BREAK_TABLE (prev_break_type));
 
               /* Note that our table assumes that combining marks
                * are only applied to alphabetic characters;
@@ -739,8 +837,6 @@ pango_default_break (const gchar   *text,
                * boundaries.
                */
 
-              g_assert (IN_BREAK_TABLE (prev_break_type));
-
               switch (break_type)
                 {
                 case G_UNICODE_BREAK_MANDATORY:
@@ -761,23 +857,45 @@ pango_default_break (const gchar   *text,
                   break_op = BREAK_ALLOWED;
                   break;
 
-                case G_UNICODE_BREAK_AMBIGUOUS:
-                  /* FIXME resolve East Asian width to figure out what to do */
-                case G_UNICODE_BREAK_COMPLEX_CONTEXT:
-                  /* FIXME language engine analysis */
-                case G_UNICODE_BREAK_UNKNOWN:
-                case G_UNICODE_BREAK_ALPHABETIC:
-                  /* treat all of the above as alphabetic for now */
-                  break_op = BREAK_OP (prev_break_type, G_UNICODE_BREAK_ALPHABETIC);
-                  break;
-
                 case G_UNICODE_BREAK_SURROGATE:
 		  g_assert_not_reached ();
                   break;
 
-                default:
-                  g_assert (IN_BREAK_TABLE (prev_break_type));
-                  g_assert (IN_BREAK_TABLE (break_type));
+		/* Hangul additions are from Unicode 4.1 UAX#14 */
+		case G_UNICODE_BREAK_HANGUL_L_JAMO:
+		case G_UNICODE_BREAK_HANGUL_V_JAMO:
+		case G_UNICODE_BREAK_HANGUL_T_JAMO:
+		case G_UNICODE_BREAK_HANGUL_LV_SYLLABLE:
+		case G_UNICODE_BREAK_HANGUL_LVT_SYLLABLE:
+                  /* treat Jamo as IDEOGRAPHIC from now
+		   */
+                  break_type = G_UNICODE_BREAK_IDEOGRAPHIC;
+
+		  if (makes_hangul_syllable)
+		    break_op = BREAK_IF_SPACES;
+		  else
+		    break_op = BREAK_ALLOWED;
+		  break;
+
+		case G_UNICODE_BREAK_AMBIGUOUS:
+		  /* FIXME:
+		   * we need to resolve the East Asian width
+		   * to decide what to do here
+		   */
+		case G_UNICODE_BREAK_COMPLEX_CONTEXT:
+		  /* FIXME:
+		   * language engines should handle this case...
+		   */
+		case G_UNICODE_BREAK_UNKNOWN:
+	          /* treat unknown, complex, and ambiguous like ALPHABETIC
+		   * for now
+	           */
+                  break_op = BREAK_OP (prev_break_type, G_UNICODE_BREAK_ALPHABETIC);
+                  break;
+
+		default:
+
+		  g_assert (IN_BREAK_TABLE (break_type));
                   break_op = BREAK_OP (prev_break_type, break_type);
                   break;
                 }
@@ -814,6 +932,7 @@ pango_default_break (const gchar   *text,
         {
           prev_break_type = break_type;
           prev_was_break_space = FALSE;
+	  prev_jamo = jamo;
         }
       else
         prev_was_break_space = TRUE;
@@ -1342,7 +1461,7 @@ pango_default_break (const gchar   *text,
       prev_type = type;
       prev_wc = wc;
 
-      /* wc might not be a valid unicode base character, but really all we
+      /* wc might not be a valid Unicode base character, but really all we
        * need to know is the last non-combining character */
       if (type != G_UNICODE_COMBINING_MARK && 
           type != G_UNICODE_ENCLOSING_MARK && 
@@ -1373,9 +1492,6 @@ pango_break (const gchar   *text,
   g_return_if_fail (analysis != NULL);
   g_return_if_fail (attrs != NULL);
   
-  if (length < 0)
-    length = strlen (text);
-
   if (analysis->lang_engine &&
       PANGO_ENGINE_LANG_GET_CLASS (analysis->lang_engine)->script_break)
     PANGO_ENGINE_LANG_GET_CLASS (analysis->lang_engine)->script_break (analysis->lang_engine, text, length, analysis, attrs, attrs_len);
@@ -1411,13 +1527,18 @@ pango_find_paragraph_boundary (const gchar *text,
   const gchar *end;
   const gchar *start = NULL;
   const gchar *delimiter = NULL;
-  gunichar prev_wc;
 
   /* Only one character has type G_UNICODE_PARAGRAPH_SEPARATOR in
-   * Unicode 3.0; update this if that changes.
+   * Unicode 4.1; update the following code if that changes.
    */
-#define PARAGRAPH_SEPARATOR 0x2029
-  
+
+  /* prev_sep is the first byte of the previous separator.  Since
+   * the valid separators are \r, \n, and PARAGRAPH_SEPARATOR, the
+   * first byte is enough to identify it.
+   */
+  gchar prev_sep;
+
+
   if (length < 0)
     length = strlen (text);
 
@@ -1432,29 +1553,21 @@ pango_find_paragraph_boundary (const gchar *text,
   if (length == 0)
     return;
 
-  /* FIXME there's plenty of room to optimize this; e.g. there's
-   * no real need to g_utf8_get_char() on every char
-   */
-  
-  prev_wc = 0;
+  prev_sep = 0;
 
   while (p != end)
     {
-      gunichar wc;
-
-      wc = g_utf8_get_char (p);
-      
-      if (prev_wc == '\n' ||
-          prev_wc == PARAGRAPH_SEPARATOR)
+      if (prev_sep == '\n' ||
+          prev_sep == PARAGRAPH_SEPARATOR_STRING[0])
         {
           g_assert (delimiter);
           start = p;
           break;
         }
-      else if (prev_wc == '\r')
+      else if (prev_sep == '\r')
         {
           /* don't break between \r and \n */
-          if (wc != '\n')
+          if (*p != '\n')
             {
               g_assert (delimiter);
               start = p;
@@ -1462,13 +1575,18 @@ pango_find_paragraph_boundary (const gchar *text,
             }
         }
       
-      if ((wc == '\n' ||
-           wc == '\r' ||
-           wc == PARAGRAPH_SEPARATOR) &&
-          delimiter == NULL)
-        delimiter = p;
+      if (*p == '\n' ||
+           *p == '\r' ||
+           !strncmp(p, PARAGRAPH_SEPARATOR_STRING,
+		    strlen(PARAGRAPH_SEPARATOR_STRING)))
+        {
+          if (delimiter == NULL)
+	    delimiter = p;
+	  prev_sep = *p;
+	}
+      else
+	prev_sep = 0;
       
-      prev_wc = wc;
       p = g_utf8_next_char (p);
     }
 
@@ -1504,7 +1622,6 @@ pango_get_log_attrs (const char    *text,
                      PangoLogAttr  *log_attrs,
                      int            attrs_len)
 {
-  int n_chars;
   PangoMap *lang_map;
   int chars_broken;
   const char *pos;
@@ -1514,7 +1631,7 @@ pango_get_log_attrs (const char    *text,
   int chars_in_range;
   static guint engine_type_id = 0;
   static guint render_type_id = 0;
-  PangoAnalysis analysis = { NULL, NULL, NULL, 0 };
+  PangoAnalysis analysis = { NULL, NULL, NULL, 0, NULL, NULL };
 
   analysis.level = level;
 
@@ -1533,14 +1650,6 @@ pango_get_log_attrs (const char    *text,
       render_type_id = g_quark_from_static_string (PANGO_RENDER_TYPE_NONE);
     }
 
-  n_chars = g_utf8_strlen (text, length);
-
-  if (attrs_len < (n_chars + 1))
-    {
-      g_warning ("pango_get_log_attrs(): length of PangoLogAttr array must be at least the number of chars in the text plus one more for the end position");
-      return;
-    }
-  
   lang_map = pango_find_map (language, engine_type_id, render_type_id);
 
   range_start = text;
