@@ -804,7 +804,7 @@ extern "C" {
   EXPORT_DEF
   FT_Error  TT_GPOS_Add_Feature( TTO_GPOSHeader*  gpos,
                                  FT_UShort        feature_index,
-                                 FT_UShort        property );
+                                 FT_UInt          property );
   EXPORT_DEF
   FT_Error  TT_GPOS_Clear_Features( TTO_GPOSHeader*  gpos );
 
