@@ -89,7 +89,8 @@ pango_script_for_unichar_bsearch (gunichar ch)
 {
   int lower = 0;
   int upper = G_N_ELEMENTS (pango_script_table) - 1;
-  static int mid = PANGO_SCRIPT_TABLE_MIDPOINT;
+  static int saved_mid = PANGO_SCRIPT_TABLE_MIDPOINT;
+  int mid = saved_mid;
 
 
   do 
@@ -99,7 +100,7 @@ pango_script_for_unichar_bsearch (gunichar ch)
       else if (ch >= pango_script_table[mid].start + pango_script_table[mid].chars)
 	lower = mid + 1;
       else
-	return pango_script_table[mid].script;
+	return pango_script_table[saved_mid = mid].script;
 
       mid = (lower + upper) / 2;
     }
@@ -118,6 +119,8 @@ pango_script_for_unichar_bsearch (gunichar ch)
  * result is undefined.
  * 
  * Return value: the #PangoScript for the character.
+ *
+ * Since: 1.4
  **/
 PangoScript
 pango_script_for_unichar (gunichar ch)
@@ -143,6 +146,8 @@ pango_script_for_unichar (gunichar ch)
  * Return value: the new script iterator, initialized
  *  to point at the first range in the text. If the string is
  *  empty, it will point at an empty range.
+ *
+ * Since: 1.4
  **/
 PangoScriptIter *
 pango_script_iter_new (const char *text,
@@ -172,6 +177,8 @@ pango_script_iter_new (const char *text,
  * @iter: a #PangoScriptIter
  * 
  * Frees a #PangoScriptIter created with pango_script_iter_new().
+ *
+ * Since: 1.4
  **/
 void
 pango_script_iter_free (PangoScriptIter *iter)
@@ -189,6 +196,8 @@ pango_script_iter_free (PangoScriptIter *iter)
  * Gets information about the range to which @iter currently points.
  * The range is the set of locations p where *start <= p < *end.
  * (That is, it doesn't include the character stored at *end)
+ *
+ * Since: 1.4
  **/
 void
 pango_script_iter_get_range (PangoScriptIter      *iter,
@@ -262,6 +271,8 @@ get_pair_index (gunichar ch)
  * is returned.
  * 
  * Return value: %TRUE if the iter was succesfully advanced.
+ *
+ * Since: 1.4
  **/
 gboolean
 pango_script_iter_next (PangoScriptIter *iter)
