@@ -24,7 +24,9 @@
 #include "pangocairo.h"
 #include "pangocairo-private.h"
 
-#if defined (HAVE_CAIRO_FREETYPE)
+#if defined (HAVE_CAIRO_ATSUI)
+#  include "pangocairo-atsui.h"
+#elif defined (HAVE_CAIRO_FREETYPE)
 #  include "pangocairo-fc.h"
 #elif defined (HAVE_CAIRO_WIN32)
 #  include "pangocairo-win32.h"
@@ -86,7 +88,9 @@ pango_cairo_font_map_new (void)
   /* Make sure that the type system is initialized */
   g_type_init ();
 
-#if defined(HAVE_CAIRO_WIN32)
+#if defined(HAVE_CAIRO_ATSUI)
+  return g_object_new (PANGO_TYPE_CAIRO_ATSUI_FONT_MAP, NULL);
+#elif defined(HAVE_CAIRO_WIN32)
   return g_object_new (PANGO_TYPE_CAIRO_WIN32_FONT_MAP, NULL);
 #elif defined(HAVE_CAIRO_FREETYPE)
   return g_object_new (PANGO_TYPE_CAIRO_FC_FONT_MAP, NULL);
