@@ -23,12 +23,6 @@
 
 G_DEFINE_ABSTRACT_TYPE (PangoFcDecoder, pango_fc_decoder, G_TYPE_OBJECT)
 
-static void
-pango_fc_decoder_init (PangoFcDecoder *decoder);
-
-static void
-pango_fc_decoder_class_init (PangoFcDecoderClass *klass);
-
 static void
 pango_fc_decoder_init (PangoFcDecoder *decoder)
 {
@@ -67,9 +61,9 @@ pango_fc_decoder_get_charset (PangoFcDecoder *decoder,
  * pango_fc_decoder_get_glyph:
  * @decoder: a #PangoFcDecoder
  * @fcfont: a #PangoFcFont to query.
- * @wc: the unicode code point to convert to a single #PangoGlyph.
+ * @wc: the Unicode code point to convert to a single #PangoGlyph.
  *
- * Generates a #PangoGlyph for the given unicode point using the
+ * Generates a #PangoGlyph for the given Unicode point using the
  * custom decoder. For complex scripts where there can be multiple
  * glyphs for a single character, the decoder will return whatever
  * glyph is most convenient for it. (Usually whatever glyph is directly
