@@ -37,11 +37,10 @@ enum {
 
 typedef struct _PangoFcFontPrivate PangoFcFontPrivate;
 
-#define PANGO_FC_FONT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), PANGO_TYPE_FC_FONT, PangoFcFontPrivate))
-
 struct _PangoFcFontPrivate
 {
   PangoFcDecoder *decoder;
+  gpointer context_key;
 };
 
 static gboolean pango_fc_font_real_has_char  (PangoFcFont *font,
@@ -102,6 +101,9 @@ pango_fc_font_class_init (PangoFcFontClass *class)
 static void
 pango_fc_font_init (PangoFcFont *fcfont)
 {
+  fcfont->priv = G_TYPE_INSTANCE_GET_PRIVATE (fcfont,
+					      PANGO_TYPE_FC_FONT,
+					      PangoFcFontPrivate);
 }
 
 static void
@@ -115,7 +117,7 @@ static void
 pango_fc_font_finalize (GObject *object)
 {
   PangoFcFont *fcfont = PANGO_FC_FONT (object);
-  PangoFcFontPrivate *priv = PANGO_FC_FONT_GET_PRIVATE (fcfont);
+  PangoFcFontPrivate *priv = fcfont->priv;
 
   g_slist_foreach (fcfont->metrics_by_lang, (GFunc)free_metrics_info, NULL);
   g_slist_free (fcfont->metrics_by_lang);  
@@ -234,7 +236,7 @@ pango_fc_font_get_coverage (PangoFont     *font,
 			    PangoLanguage *language)
 {
   PangoFcFont *fcfont = (PangoFcFont *)font;
-  PangoFcFontPrivate *priv = PANGO_FC_FONT_GET_PRIVATE (fcfont);
+  PangoFcFontPrivate *priv = fcfont->priv;
   FcCharSet *charset;
 
   if (priv->decoder)
@@ -572,7 +574,7 @@ gboolean
 pango_fc_font_has_char (PangoFcFont *font,
 			gunichar     wc)
 {
-  PangoFcFontPrivate *priv = PANGO_FC_FONT_GET_PRIVATE (font);
+  PangoFcFontPrivate *priv = font->priv;
   FcCharSet *charset;
 
   g_return_val_if_fail (PANGO_IS_FC_FONT (font), FALSE);
@@ -604,12 +606,7 @@ PangoGlyph
 pango_fc_font_get_glyph (PangoFcFont *font,
 			 gunichar     wc)
 {
-  PangoFcFontPrivate *priv;
-  
-  g_return_val_if_fail (PANGO_IS_FC_FONT (font), 0);
-
-  priv = PANGO_FC_FONT_GET_PRIVATE (font);
-
+  PangoFcFontPrivate *priv = font->priv;
 
   /* Replace NBSP with a normal space; it should be invariant that
    * they shape the same other than breaking properties.
@@ -715,7 +712,7 @@ pango_fc_font_kern_glyphs (PangoFcFont      *font,
 PangoFcDecoder *
 _pango_fc_font_get_decoder (PangoFcFont *font)
 {
-  PangoFcFontPrivate *priv = PANGO_FC_FONT_GET_PRIVATE (font);
+  PangoFcFontPrivate *priv = font->priv;
 
   return priv->decoder;
 }
@@ -735,7 +732,7 @@ void
 _pango_fc_font_set_decoder (PangoFcFont    *font,
 			    PangoFcDecoder *decoder)
 {
-  PangoFcFontPrivate *priv = PANGO_FC_FONT_GET_PRIVATE (font);
+  PangoFcFontPrivate *priv = font->priv;
 
   if (priv->decoder)
     g_object_unref (priv->decoder);
@@ -746,6 +743,23 @@ _pango_fc_font_set_decoder (PangoFcFont    *font,
     g_object_ref (priv->decoder);
 }
 
+gpointer
+_pango_fc_font_get_context_key (PangoFcFont *fcfont)
+{
+  PangoFcFontPrivate *priv = fcfont->priv;
+
+  return priv->context_key;
+}
+
+void
+_pango_fc_font_set_context_key (PangoFcFont *fcfont,
+				gpointer     context_key)
+{
+  PangoFcFontPrivate *priv = fcfont->priv;
+
+  priv->context_key = context_key;
+}
+
 static FT_Glyph_Metrics *
 get_per_char (FT_Face      face,
 	      FT_Int32     load_flags,
@@ -780,6 +794,8 @@ get_per_char (FT_Face      face,
  * The other possibility would be to to make it public in something
  * like it's current form, and also expose glyph information
  * caching functionality similar to pango_ft2_font_set_glyph_info().
+ *
+ * Since: 1.6
  **/
 void
 pango_fc_font_get_raw_extents (PangoFcFont    *fcfont,
