@@ -159,7 +159,7 @@ static void
 pango_fc_font_map_init (PangoFcFontMap *fcfontmap)
 {
   static gboolean registered_modules = FALSE;
-  PangoFcFontMapPrivate *priv = fcfontmap->priv;
+  PangoFcFontMapPrivate *priv;
 
   priv = fcfontmap->priv = G_TYPE_INSTANCE_GET_PRIVATE (fcfontmap,
 							PANGO_TYPE_FC_FONT_MAP,
@@ -483,6 +483,9 @@ pango_fc_font_map_finalize (GObject *object)
   g_queue_free (priv->fontset_cache);
   g_hash_table_destroy (priv->coverage_hash);
 
+  if (priv->fontset_hash)
+    g_hash_table_destroy (priv->fontset_hash);
+
   if (priv->font_hash)
     g_hash_table_destroy (priv->font_hash);
 
@@ -553,7 +556,7 @@ pango_fc_font_map_add (PangoFcFontMap *fcfontmap,
   key.pattern = fcfont->font_pattern;
   
   key_copy = font_hash_key_copy (&key);
-  fcfont->context_key = key_copy->context_key;
+  _pango_fc_font_set_context_key (fcfont, key_copy->context_key);
   fcfont->matrix = key.matrix;
   
   g_hash_table_insert (priv->font_hash, key_copy, fcfont);
@@ -584,11 +587,11 @@ _pango_fc_font_map_remove (PangoFcFontMap *fcfontmap,
   key.fontmap = fcfontmap;
   key.matrix = fcfont->matrix;
   key.pattern = fcfont->font_pattern;
-  key.context_key = fcfont->context_key;
+  key.context_key = _pango_fc_font_get_context_key (fcfont);
   
   g_hash_table_remove (priv->font_hash, &key);
   fcfont->fontmap = NULL;
-  fcfont->context_key = NULL;
+  _pango_fc_font_set_context_key (fcfont, NULL);
   g_object_unref (fcfontmap);
 }
 
@@ -1008,44 +1011,6 @@ get_unscaled_size (PangoFcFontMap             *fcfontmap,
     }
 }
 
-/*
- * Based on cairo-matrix.c:_cairo_matrix_compute_scale_factors()
- *
- * Copyright 2005, Keith Packard
- */
-static int
-get_scaled_size (FontsetHashKey *key)
-{
-  PangoMatrix *matrix = &key->matrix;
-  double det = matrix->xx * matrix->yy - matrix->yx * matrix->xy;
-
-  if (det == 0)
-    {
-      return 0.0;
-    }
-  else
-    {
-      double x = matrix->xx;
-      double y = matrix->yx;
-      double major, minor;
-
-      major = sqrt (x*x + y*y);
-      
-      /*
-       * ignore mirroring
-       */
-      if (det < 0)
-	det = - det;
-      
-      if (major)
-	minor = det / major;
-      else 
-	minor = 0.0;
-
-      return minor * (key->size / 1024.);
-    }
-}
-
 static PangoFcPatternSet *
 pango_fc_font_map_get_patterns (PangoFontMap               *fontmap,
 				PangoContext               *context,
@@ -1080,8 +1045,10 @@ pango_fc_font_map_get_patterns (PangoFontMap               *fontmap,
 
   if (patterns == NULL)
     {
+      double scale_factor = pango_matrix_get_font_scale_factor (&key.matrix);
+      double scaled_size = key.size * scale_factor / PANGO_SCALE;
       pattern = pango_fc_make_pattern (desc, language,
-				       get_scaled_size (&key),
+				       scaled_size,
 				       pango_fc_font_map_get_resolution (fcfontmap, context));
       
       pango_fc_default_substitute (fcfontmap, context, pattern);
