@@ -96,6 +96,12 @@ _pango_xft_font_new (PangoXftFontMap *xftfontmap,
   xfont = (PangoXftFont *)g_object_new (PANGO_TYPE_XFT_FONT,
 					"pattern", pattern,
 					NULL);
+
+  /* Hack to force hinting of vertical metrics; hinting off for
+   * a Xft font just means to not hint outlines, but we still
+   * want integer line spacing, underline positions, etc
+   */
+  PANGO_FC_FONT (xfont)->is_hinted = TRUE;
   
   xfont->xft_font = NULL;
 
@@ -123,14 +129,19 @@ _pango_xft_font_get_mini_font (PangoXftFont *xfont)
   if (!xfont->mini_font)
     {
       Display *display;
+      int screen;
       PangoFontDescription *desc = pango_font_description_new ();
+      PangoContext *context;
       int i;
       int width = 0, height = 0;
       XGlyphInfo extents;
       XftFont *mini_xft;
       int new_size;
       
-      _pango_xft_font_map_get_info (fcfont->fontmap, &display, NULL);
+      _pango_xft_font_map_get_info (fcfont->fontmap, &display, &screen);
+
+      context = pango_xft_get_context (display, screen);
+      pango_context_set_language (context, pango_language_from_string ("en"));
 
       pango_font_description_set_family_static (desc, "monospace");
 
@@ -141,8 +152,10 @@ _pango_xft_font_get_mini_font (PangoXftFont *xfont)
       else
 	pango_font_description_set_size (desc, new_size);
 
-      xfont->mini_font = pango_font_map_load_font (fcfont->fontmap, NULL, desc);
+      xfont->mini_font = pango_font_map_load_font (fcfont->fontmap, context, desc);
+
       pango_font_description_free (desc);
+      g_object_unref (context);
       
       mini_xft = xft_font_get_font (xfont->mini_font);
       
@@ -352,7 +365,7 @@ load_fallback_font (PangoXftFont *xfont)
   _pango_xft_font_map_get_info (fcfont->fontmap, &display, &screen);
   
   size_is_absolute = pango_font_description_get_size_is_absolute (fcfont->description);
-  size = (double)pango_font_description_get_size (fcfont->description) / PANGO_SCALE;
+  size = pango_font_description_get_size (fcfont->description) / PANGO_SCALE;
       
   xft_font = XftFontOpen (display,  screen,
                           FC_FAMILY, FcTypeString, "sans",
@@ -561,14 +574,14 @@ pango_xft_font_unlock_face (PangoFont *font)
  * @font: a #PangoFont for the Xft backend
  * @wc: Unicode codepoint to look up
  * 
- * Gets the glyph index for a given unicode codepoint
+ * Gets the glyph index for a given Unicode character
  * for @font. If you only want to determine
  * whether the font has the glyph, use pango_xft_font_has_char().
  * 
  * Use pango_fc_font_get_glyph() instead.
  *
- * Return value: the glyph index, or 0, if the unicode
- *  codepoint doesn't exist in the font.
+ * Return value: the glyph index, or 0, if the Unicode
+ *  character does not exist in the font.
  *
  * Since: 1.2
  **/
